#include <stdio.h>
#include <math.h>
#include <string.h>

/*  Loads a 8192 sample long section of a sound data file
    and Fourier Transforms it to produce the power/frequency
    spectrum. Averages over a series of spectra.
    Reads WAV LPCM format data
    
    J.C.G.Lesurf June 2010
    
     */

#define SWAP(a,b) tempr=(a) ; (a)=(b) ; (b)=tempr

FILE* infile;
FILE* rfile;
FILE* outfile;
FILE* lffile;

unsigned char ChunkID[5],Format[5],Subchunk1ID[5],Subchunk2ID[5];

long ChunkSize,Subchunk1Size,AudioFormat;
long NumChannels,SampleRate,ByteRate,BlockAlign;
long BitsPerSample,Subchunk2Size,BytesPerSample;

long pairs_per_sec,ifl,ileaf;
long irx;

long ilength,offset,block_number,samples_per_block;
long nn,n,npoints,nhalf,nspectra,spectrum_number,ip,ipk;
long harm,apodising,infileok,ok,bitsend;
long nbit,nperbit,bitcount;


double tstart,tstop,tbit,block_duration,total_time,t,dt,f,f0,pi2,pi,npi,sn;

double pmean_left,pmean_right,fpeak,ppeak,psum,tnow;
double ppeak_left,ppeak_right,thd_left,thd_right;
double fcount,spec_scale,required_duration,dc_left,dc_right;
double dphi0,fscale,fi0,scale_left,scale_right,rms_left,rms_right;
double fcount,fbit;

unsigned char tc,lchar;
unsigned char apstring[16];

unsigned char datablock[49152];
double left[8192], right[8192],weight[8192];
double x[16385];
double spec_out[4096];

double phi[8193],dphi[8193],fir[8193];

unsigned char infilename[512];
unsigned char pathname[512];
unsigned char leafname[128];
unsigned char outfilename[512];
unsigned char rfilename[512];
unsigned char appdirname[512];
unsigned char resultsdirname[512];
unsigned char firname[512];
unsigned char hfname[512];
unsigned char vrelhfname[512];
unsigned char vrellfname[512];
unsigned char vrelrepname[512];

void read_settings_file(void);

long get_start_time(void);
long get_stop_time(void);
void load_block(void);
void load_data_block(void);
void load_data_block24(void);
void zero_spectra(void);
void transform(double*);
void scale_block(void);
void get_phase(void);
void stitch_phase(void);

void load_fft_array(double*);
void save_powerspectrum(double*,double*);
double db(double);
double db0(double);
void fft(void);
void bit_rev(void);
void save_output(void);
void analyse_onespectrum(void);
void save_spectrum(double*);
void update_bit(double);
void get_fi0(void);
void get_fir_ppm(void);
void report_phi(void);

void process_lf(void);
void save_lf_output(void);
void save_lf_spectrum(void);

void generate_apodise_weights(void);
void apodise(double*);

double find_counter_frequency(void);

double read_header(void);
void get4(unsigned char*);
long get4int(void);
long get2int(void);

void veusz_lf_spectrum_output(void);
void veusz_hf_spectrum_output(void);
void veusz_lf_fir_output(void);
void veusz_label(char*,char*,double,double);

int main(int argc,char* argv[])
{
  samples_per_block=8192;                   /* Number of samples per channel per block */
  spec_scale=1.0/(double)samples_per_block;
  n=samples_per_block; npoints=n;
  nn=2*n;  nhalf=n/2;
  pi=3.14159265359; npi=(-pi); pi2=2.0*pi;
  nspectra=8;
  
  printf("  WAV IQ FFTScan Ver 1.01 13th June 2011\n");
  printf("  ======================================\n\n");
  printf("Does an analysis of an IQ waveform in a chosen section of a WAV file\n");
  printf("Please read the Help inside the application before first use.\n");
  
  sprintf(appdirname,"%s\0",argv[0]);
  ok=(long)strrchr(appdirname,'/')-(long)appdirname;
  appdirname[ok]=(char)0;
  printf("Application directory = %s\n",appdirname);
  
  sprintf(resultsdirname,"%s/Results\0",appdirname);
  printf("Results will be written into the directory =\n  %s\n\n",resultsdirname);
  
  
  
  if (argc>1)
  {
    infileok=1;
    sprintf(infilename,"%s\0",argv[1]);
    printf("Input from file = %s\n\n",infilename);
    
    ok=(long)strrchr(infilename,'/')-(long)infilename;
    ok++;
    ileaf=0;
    do
    {
      lchar=infilename[ok];
      if (lchar=='.')
      {
        lchar=(char)0;
      }
      
      leafname[ileaf]=lchar;
      
      ileaf++; ok++;
    } while (lchar!=(char)0);
    
    printf("Leaf name = %s\n\n",leafname);
        
    
    
    
  }
  else
  {
    infileok=0;
    printf("Drag a WAV file and drop it on the application icon\n");
    printf("if you wish to examine the contents of the file.\n\n");
    total_time=0.0;
  }
  
  
  if (infileok==1)
  {
    printf("Now reading WAV header\n");
    total_time=read_header();
    printf("total time = %6.2lf\n",total_time);
    read_settings_file();
    dt=1.0/(double)pairs_per_sec;                 /* sampling interval */
    block_duration=dt*(double)samples_per_block;  /* Duration of block in seconds */
    
    required_duration=8192.0*tbit;
    printf("Duration required for LF analysis = %6.2lf sec [ %6.2lf msec res]\n",required_duration,tbit*1000.0);
    
    if(total_time<(tstart+required_duration+2.0))
    {
      printf(" * * *File not long enough! * * *\n");
      total_time=0.0;
    }              
  }
  
  
  if(total_time>0.0)
  {
    printf("OK\n");
    
    apodising=1;
    generate_apodise_weights();
    spectrum_number=0;
  
    zero_spectra();
    
    printf("Starting analysis\n");
    sprintf(firname,"%s/fir_ppm_%s.csv\0",resultsdirname,leafname);
    sprintf(vrelrepname,"./fir_ppm_%s.csv\0",leafname);
    printf("relative name [vrelrepname] = %s\n",vrelrepname);
    
    printf("Opening %s\n",firname);
    lffile=fopen(firname,"wt");
    fprintf(lffile,"\"t sec\", \"ppm\",\n");
    f0=1.0/(8192.0*dt);
    
    nperbit=(long)(tbit/dt);
    
    printf("  %ld samples per %6.2lf ms LF chunk\n",nperbit,tbit*1000.0);
    nbit=0; bitcount=0; fbit=0.0;
    offset=get_start_time();
    
    sn=(double)nspectra; sn=2.0/sn;
    printf("Opening %s\n",infilename);
    
    infile=fopen(infilename,"rd");
    fseek(infile,offset+44,SEEK_SET);
    
    load_block();
    
    printf("First block rms = %8.5lf left  %8.5lf right\n",rms_left,rms_right);
    printf("            dc  = %8.5lf left  %8.5lf right\n",dc_left,dc_right);
    scale_left=1.0/rms_left;  scale_right=1.0/rms_right;
    printf(" scale factors  = %8.5lf left  %8.5lf right\n",scale_left,scale_right);
    
    scale_block();
    get_phase();
    dphi[0]=dphi[1]; /* fill in first dphi to cover lack of carry over */
    stitch_phase();
    
    get_fi0();
    
    printf("Mean IQ frequency = %10.6lf Hz\n",fi0);
    
    fscale=1.0E6/(fi0*pi2*dt);
    
    get_fir_ppm();
    
    report_phi();
    transform(fir);
    spectrum_number++;

    do
    {
      
      load_block();
      
      printf("uncorrected rms = %8.5lf left  %8.5lf right\n",rms_left,rms_right);
      printf("            dc  = %8.5lf left  %8.5lf right\n",dc_left,dc_right);
      
      scale_block();
      get_phase();
      stitch_phase();
      get_fir_ppm();
      
      if(spectrum_number<4)
      {
        report_phi();
      }
      transform(fir);
      spectrum_number++;
      
    } while (spectrum_number<nspectra);
    
    save_output();
    
    printf("Scanning for LF analysis\n\n");
    
    do
    {
      
      load_block();
      /*
      printf("uncorrected rms = %8.5lf left  %8.5lf right\n",rms_left,rms_right);
      printf("            dc  = %8.5lf left  %8.5lf right\n",dc_left,dc_right);
      */
      scale_block();
      get_phase();
      stitch_phase();
      get_fir_ppm();
      
      
      
      
    } while (bitcount<bitsend);
    
    
    
    
    fclose(infile);
    fclose(lffile);
    
    process_lf();
    
    veusz_lf_spectrum_output();
    veusz_lf_fir_output();
    veusz_hf_spectrum_output();
    
    printf("Analysis done!\n");
  }
  
  return 0;
}

void process_lf(void)
{
  long ipl,icheck,istop;
  double dum,vin;
  unsigned char igo;
  ipl=0; istop=0;
  
  infile=fopen(firname,"rt");
  
  do
  {
    icheck=(long)fscanf(infile,"%c",&igo);
    
  } while (15<=(int)igo);
  
  do
  {
    icheck=(long)fscanf(infile,"%lf,%lf,",&dum,&vin);
    if(icheck==2)
    {
      fir[ipl]=vin;
      ipl++;
    }
    else
    {
      istop=1;
    }
    
    if(ipl>=8192)
    {
      istop=2;
    }
    
  } while (istop==0);
  
  fclose(infile);
  
  printf("istop  %ld with %ld  values read\n",istop,ipl);
  
  if(istop==1)
  {
    printf(" *** Input file not long enough or not 2 values per line! ***\n");
  }
  else
  {
    if(ipl==8192)
    {
      printf("Now doing LF spectrum\n");
      
      f0=1.0/(8192.0*tbit);
      
      apodise(fir);
      load_fft_array(fir);
      fft();
      save_lf_spectrum();
      save_lf_output();
    }
  }
}

void save_lf_spectrum(void)
{
  long icount,idc;
  double p1,p2;
  icount=0; idc=1;
  do
  {
    p1=x[idc];
    idc++;
    p2=x[idc];
    idc++;
    p1=p1*p1+p2*p2;
    
    p1=spec_scale*sqrt(p1);
    
    fir[icount]=p1;
    icount++;
    
  } while (icount<nhalf);
  
}

void save_lf_output(void)
{
  long q;
  double f;
  
  sprintf(outfilename,"%s/LFspectrum_ppb_%s.csv\0",resultsdirname,leafname);
  sprintf(vrellfname,"./LFspectrum_ppb_%s.csv\0",leafname);
  printf("Setting vrellfname = %s\n",vrellfname);
  
  
  outfile=fopen(outfilename,"wt");
  
  fprintf(outfile,"\"f Hz\", \"ppb\",\n");
  
  q=0;
  do
  {
    f=f0*(double)q;
    
    fprintf(outfile,"%8.5lf,  %8.4lf,\n",f,2000.0*fir[q]);
    q++;
  } while (q<nhalf);
  
  fclose(outfile);
  
  printf("\nLF spectrum done!\n");
}

void save_output(void)
{
  long q;
  double f;
  
  sprintf(outfilename,"%s/HFspectrum_ppm_%s_ave%02ld.csv\0",resultsdirname,leafname,nspectra);
  sprintf(hfname,"%s\0",outfilename);
  
  sprintf(vrelhfname,"./HFspectrum_ppm_%s_ave%02ld.csv\0",leafname,nspectra);
  printf("Setting vrelhfname = %s\n",vrelhfname);
  
  outfile=fopen(outfilename,"wt");
  fprintf(outfile,"\"f Hz\", \"ppm\",\n");
  
  q=1;
  do
  {
    f=f0*(double)q;
    
    fprintf(outfile,"%8.2lf,  %8.4lf,\n",f,sn*spec_out[q]);
    q++;
  } while (q<nhalf);
  
  fclose(outfile);
  
  printf("\nHF spectrum done!\n");
}

long get_start_time(void)
{
  /* This procedure works out the offset for the
    chosen start time */
  
  long ir,ir4;
  double irtemp;
  ir=0;
  irtemp=(double)pairs_per_sec*tstart;
  ir4=(long)(4.0*irtemp);
  ir=4*(long)irtemp;
  printf("Start sample %8.2lf  %ld  %ld\n",irtemp,ir4,ir);
  return ir;
}
void load_block(void)
{
  if(BytesPerSample==2)
  {
    load_data_block();
  }
  if(BytesPerSample==3)
  {
    load_data_block24();
  }
}

void load_data_block(void)
{
  
  /* This procedure finds and loads the selected block of sample
     data and then converts the raw data into a set of real sample
     values for analysis. */
  
  long block_size_read;
  long icut,icut2;
  long io,ii,lt,rt;
  double samscale;
  
  icut=32768; icut2=icut*2;
  samscale=1.0/(double)icut;
  dc_left=0.0; dc_right=0.0;
  rms_left=0.0; rms_right=0.0;
  block_size_read=fread(datablock,sizeof(unsigned char),32768,infile);
  ii=0; io=0;
  
  do
  {
    lt=  256*(long)datablock[ii+1] + (long)datablock[ii];
    ii+=2;
    if(NumChannels==2)
    {
      rt= 256*(long)datablock[ii+1] + (long)datablock[ii];
      ii+=2;
    }
    else
    {
      rt=lt;
    }
    if (lt>=icut) lt-=icut2;
    if (rt>=icut) rt-=icut2;
    left[io]=samscale*(double)lt;
    right[io]=samscale*(double)rt;
    rms_right+=right[io]*right[io];
    rms_left+=left[io]*left[io];
    
    io++;
    
  } while (io<8192);
  
  dc_right=dc_right/8192.0;
  dc_left=dc_left/8192.0;
  rms_left=rms_left/8192.0;
  rms_right=rms_right/8192.0;
  
  pmean_left=rms_left;
  pmean_right=rms_right;
  
  rms_left=sqrt(rms_left);
  rms_right=sqrt(rms_right);
  
  
  
}

void load_data_block24(void)
{
  
  /* This procedure finds and loads the selected block of sample
     data and then converts the raw data into a set of real sample
     values for analysis. */
  
  long block_size_read;
  long icut,icut2;
  long io,ii,lt,rt;
  double samscale;
  
  icut=32768*256; icut2=icut*2;
  samscale=1.0/(double)icut;
  dc_left=0.0; dc_right=0.0;
  rms_left=0.0; rms_right=0.0;
  block_size_read=fread(datablock,sizeof(unsigned char),49152,infile);
  ii=0; io=0;
  
  do
  {
    lt=65536*(long)datablock[ii+2] + 256*(long)datablock[ii+1] + (long)datablock[ii];
    ii+=3;
    if(NumChannels==2)
    {
      rt=65536*(long)datablock[ii+2] + 256*(long)datablock[ii+1] + (long)datablock[ii];
      ii+=3;
    }
    else
    {
      rt=lt;
    }
    if (lt>=icut) lt-=icut2;
    if (rt>=icut) rt-=icut2;
    left[io]=samscale*(double)lt;
    right[io]=samscale*(double)rt;
    rms_right+=right[io]*right[io];
    rms_left+=left[io]*left[io];
    
    io++;
    
  } while (io<8192);
  
  dc_right=dc_right/8192.0;
  dc_left=dc_left/8192.0;
  rms_left=rms_left/8192.0;
  rms_right=rms_right/8192.0;
  
  pmean_left=rms_left;
  pmean_right=rms_right;
  
  rms_left=sqrt(rms_left);
  rms_right=sqrt(rms_right);
  
  
  
}

void scale_block(void)
{
  long iskb;
  iskb=0;
  do
  {
    left[iskb]=scale_left*left[iskb];
    right[iskb]=scale_right*right[iskb];
    iskb++;
  } while (iskb<8192);
  
}

void get_phase(void)
{
  long igp;
  double answer;
  igp=0;
  
  phi[0]=phi[8192];  /* does carryover */
  
  do
  {
    answer=atan2(left[igp],right[igp]);
    phi[igp+1]=answer;
    dphi[igp]=answer-phi[igp];
    
    igp++;
  } while (igp<8192);
    
}

void stitch_phase(void)
{
  long isph;
  double answer;
  isph=0;
  do
  {
    answer=dphi[isph];
    
    if(answer<npi)
    {
      answer+=pi2;
      dphi[isph]=answer;
    }
    
    if(answer>pi)
    {
      answer-=pi2;
      dphi[isph]=answer;
    }
    isph++;
  } while (isph<8192);
}

void get_fi0(void)
{
  long gi;
  double answer;
  answer=0.0; gi=0;
  do
  {
    answer+=dphi[gi];
    gi++;
  } while (gi<8192);
  answer=answer/8192.0;
  dphi0=answer;
  answer=answer/(pi2*dt);
  fi0=answer;
}

void get_fir_ppm(void)
{
  long gfm;
  double answer;
  gfm=0;
  do
  {
    answer=dphi[gfm];
    answer-=dphi0;
    answer=answer*fscale;
    fir[gfm]=answer;
    update_bit(answer);
    gfm++;
  } while (gfm<8192);
  
  
}

void update_bit(double ain)
{
  fbit+=ain;
  nbit++;
  
  if (nbit==nperbit)
  {
    nbit=0;
    fbit=fbit/(double)nperbit;
    fprintf(lffile,"%8.5lf,  %8.4lf,\n",tbit*(double)bitcount,fbit);
    bitcount++;
    /* printf("%4ld of %4ld time %8.4lf sec fir = %6.4lf ppm\n",bitcount,bitsend,tbit*(double)bitcount,fbit); */
    fbit=0.0;
  }
}

void bit_rev(void)
{
  long ibr,jbr,mbr;
  double tr,ti;
  jbr=1;
  ibr=1;
  do
  {
    if ( jbr > ibr )
    {
      tr=x[jbr];
      ti=x[jbr+1];
      x[jbr]=x[ibr];
      x[jbr+1]=x[ibr+1];
      x[ibr]=tr;
      x[ibr+1]=ti;
    }
    mbr=nn/2;
    while ( mbr >= 2 && jbr > mbr )
    {
      jbr-=mbr;
      mbr=mbr/2;
    }
    jbr=jbr+mbr;   
    ibr+=2;
  } while ( ibr < nn );
}

void report_phi(void)
{
  char rpname[512];
  
  sprintf(rpname,"%s/phiout_%02ld.csv\0",resultsdirname,spectrum_number);
  
  rfile=fopen(rpname,"wt");
    irx=0;
    do
    {
      tnow=1000.0*dt*(double)(irx+8192*spectrum_number);
      
      
      fprintf(rfile,"%10.5lf,%8.4lf,%8.4lf,%10.4lf\n",tnow,phi[irx],dphi[irx],fir[irx]);
      irx++;
    } while (irx<8192);
    fclose(rfile);
}

void transform(double* inarray)
{
  printf("\n\nSpectrum %ld of %ld\n",spectrum_number+1,nspectra);
  apodise(inarray);
  load_fft_array(inarray);
  fft();
  save_spectrum(spec_out);
}

void apodise(double* inarray)
{
  long iap;
  iap=0;
  do
  {
    inarray[iap]=weight[iap]*inarray[iap];
    iap++;
  } while (iap<8192);
}

void load_fft_array(double* inarray)
{
  long icount,icd;
  icount=0; icd=1;
  x[0]=0.0;
  do
  {
    
    x[icd]=inarray[icount];
    icd++;
    x[icd]=0.0;
    icd++;
    icount++;
  } while (icount<npoints);
  
  
}

void fft(void)
{
  long mmax,istep,m,i,j;
  long ibig,ismall,jbig,jsmall;
  double theta,wpi,wpr;
  double wi,wr,tr,ti,wtemp;
  
  ibig=-99;  jbig=-99;
  ismall=99; jsmall=99;

  mmax=2;
  theta=2.0*pi/mmax;
  
  bit_rev();
  
  while ( nn > mmax )
  {
   istep=2*mmax;
   theta=2.0*pi/mmax;
   wpr=-2.0*sin(0.5*theta)*sin(0.5*theta);
   wpi=sin(theta);
   wr=1.0;
   wi=0.0;
   m=1;
   do
   {
     i=m;
     do
     {
       j=i+mmax;
       
       if(i>ibig)
       {
         ibig=i;
       }
       if(i<ismall)
       {
         ismall=i;
       }
       if (j>jbig)
       {
         jbig=j;
       }
       if (j<jsmall)
       {
         jsmall=j;
       }
       
       
       tr=wr*x[j]-wi*x[j+1];
       ti=wr*x[j+1]+wi*x[j];
       x[j]=x[i]-tr;
       x[j+1]=x[i+1]-ti;
       x[i]=x[i]+tr;
       x[i+1]=x[i+1]+ti;
       i+=istep;
     } while ( i < nn );
     wtemp=wr;
     wr=wr*wpr-wi*wpi+wr;
     wi=wi*wpr+wtemp*wpi+wi;
     m+=2;
   } while ( m < mmax );
   mmax=istep;   
  }
  
}

void save_spectrum(double* outarray)
{
  long icount,idc;
  double p1,p2;
  icount=0; idc=1;
  do
  {
    p1=x[idc];
    idc++;
    p2=x[idc];
    idc++;
    p1=p1*p1+p2*p2;
    p1=spec_scale*sqrt(p1);
    outarray[icount]=outarray[icount]+p1;
    icount++;
  } while (icount<nhalf);
}




void read_settings_file(void)
{
  
  /* This proceedure reads the 'settings' file. */
  
  FILE *setfile; 
  unsigned char setfilename[512];
  sprintf(setfilename,"%s/Settings\0",appdirname);
  printf("Reading settings from %s\n\n",setfilename);
  setfile=fopen(setfilename,"rt");  
  fscanf(setfile,"%ld",&nspectra);
  printf(" Number of HF spectra to collect = %ld\n",nspectra);
  fscanf(setfile,"%ld",&bitsend);
  printf(" Number of LF fir values to collect = %ld\n",bitsend);
  fscanf(setfile,"%lf",&tbit); 
  printf("Time per LF block = %6.2lf ms\n",tbit*1000.0);
  fscanf(setfile,"%lf",&tstart);
  printf("Start at %6.2lf sec\n\n",tstart);
  
  fclose(setfile);
}

void generate_apodise_weights(void)
{
  long iwf,iwb;
  double weight_now,wscale,xplace;
  iwf=0; iwb=8191; wscale=0.0;
  printf("BH window\n");
 
  do
  {
    xplace=(double)iwf;
    xplace=pi2*xplace/8191.0;
    weight_now= 0.35875-0.48829*cos(xplace)+0.14128*cos(2.0*xplace)-0.01168*cos(3.0*xplace);
    weight[iwf]=weight_now;
    weight[iwb]=weight_now;
    wscale+=weight_now;
    iwf++;
    iwb--;
  } while (iwf<iwb);
    
  wscale=wscale/((double)(iwf-1));
    
  iwf=0;
  do
  {
    weight[iwf]=weight[iwf]/wscale;
    iwf++;
  } while (iwf<8192);
} 
  
void zero_spectra(void)
{
  long iz;
  iz=0;
  do
  {
    spec_out[iz]=0.0;
    iz++;
  } while (iz<4096);
}  

void veusz_lf_fir_output(void)
{
  unsigned char vinfile[512];
  unsigned char voutfile[512];
  unsigned char vlabeltext[256];
  unsigned char vchar;
  long vcheck,vcount;
  vcount=0;
  sprintf(vinfile,"%s/iqreport_style.vsz\0",appdirname);
  
  sprintf(voutfile,"%s/Results/load_iqreport_%s.vsz\0",appdirname,leafname);
  printf("Making veusz file %s\n",voutfile);
  printf(" using style from %s\n",vinfile);
  printf(" reading from%s\n",firname);
  
  outfile=fopen(voutfile,"wt");
  fprintf(outfile,"# Veusz saved document (version 1.7)\n\n");
  
  fprintf(outfile,"ImportFileCSV(u'%s', linked=True)\n",vrelrepname);
  
  infile=fopen(vinfile,"rt");
  
  do
  {
    vcheck=(long)fscanf(infile,"%c",&vchar);
    
    if (vcheck==1) fprintf(outfile,"%c",vchar);
    vcount++;
    
    if(vcount>10000) vcheck=0;
  } while (vcheck==1);
  
  sprintf(vlabeltext,"Low Frequency wow/flutter of   %s\0",leafname);
  veusz_label(vlabeltext,"label1\0",0.14,0.91);
  
  
  fclose(infile);
  
  fclose(outfile);
  
  
}

void veusz_hf_spectrum_output(void)
{
  unsigned char vinfile[512];
  unsigned char voutfile[512];
  unsigned char vlabeltext[256];
  unsigned char vchar;
  long vcheck,vcount;
  vcount=0;
  sprintf(vinfile,"%s/iqhfspectrum_style.vsz\0",appdirname);
  
  sprintf(voutfile,"%s/Results/load_iqhfspectrum_%s.vsz\0",appdirname,leafname);
  printf("Making veusz file\n  %s\n",voutfile);
  printf(" using style from\n  %s\n",vinfile);
  
  
  outfile=fopen(voutfile,"wt");
  fprintf(outfile,"# Veusz saved document (version 1.7)\n\n");
  
  fprintf(outfile,"ImportFileCSV(u'%s', linked=True)\n",vrelhfname);
  
  infile=fopen(vinfile,"rt");
  
  do
  {
    vcheck=(long)fscanf(infile,"%c",&vchar);
    
    if (vcheck==1) fprintf(outfile,"%c",vchar);
    vcount++;
    
    if(vcount>10000) vcheck=0;
  } while (vcheck==1);
  
  sprintf(vlabeltext,"High Frequency wow/flutter spectrum of   %s\0",leafname);
  veusz_label(vlabeltext,"label1\0",0.14,0.91);
  
  sprintf(vlabeltext,"Average over %ld fft spectra\0",nspectra);
  veusz_label(vlabeltext,"label2\0",0.14,0.86);
  
  fclose(infile);
  
  fclose(outfile);
  
  
  
}

void veusz_lf_spectrum_output(void)
{
  unsigned char vinfile[512];
  unsigned char voutfile[512];
  unsigned char vlabeltext[256];
  unsigned char vchar;
  long vcheck,vcount;
  vcount=0;
  sprintf(vinfile,"%s/iqspectrum_style.vsz\0",appdirname);
  
  sprintf(voutfile,"%s/Results/load_iqspectrum_%s.vsz\0",appdirname,leafname);
  printf("Making veusz file\n  %s\n",voutfile);
  printf(" using style from\n  %s\n",vinfile);
  
  outfile=fopen(voutfile,"wt");
  fprintf(outfile,"# Veusz saved document (version 1.7)\n\n");
  
  fprintf(outfile,"ImportFileCSV(u'%s', linked=True)\n",vrellfname);
  
  infile=fopen(vinfile,"rt");
  
  do
  {
    vcheck=(long)fscanf(infile,"%c",&vchar);
    
    if (vcheck==1) fprintf(outfile,"%c",vchar);
    vcount++;
    
    if(vcount>10000) vcheck=0;
  } while (vcheck==1);
  
  sprintf(vlabeltext,"Low Frequency wow/flutter spectrum of   %s\0",leafname);
  veusz_label(vlabeltext,"label1\0",0.14,0.91);
  
  sprintf(vlabeltext,"Start %6.2lf sec : Duration %6.2lf sec\0",tstart,required_duration);
  veusz_label(vlabeltext,"label2\0",0.14,0.86);
  
  fclose(infile);
  
  fclose(outfile);
  
}


void veusz_label(char* label_text,char* label_name,double xlpos,double ylpos)
{
   fprintf(outfile,"To('..')\n");
   fprintf(outfile,"Add('label', name='%s',autoadd=False)\n",label_name);
   fprintf(outfile,"To('%s')\n",label_name);
   fprintf(outfile,"Set('label', u'%s')\n",label_text);
   fprintf(outfile,"Set('xPos', [%7.4lf])\n",xlpos);
   fprintf(outfile,"Set('yPos', [%7.4lf])\n",ylpos);

}



  




double read_header(void)
{
  /* This procedure reads the header if it is a WAV file */
  
  infile=fopen(infilename,"rd");
  
  double result;


  get4(ChunkID);
  printf("ChunkID = %s\n",ChunkID);
  ChunkSize=get4int();
  printf("Chunk Size = %ld\n",ChunkSize);

  get4(Format);
  printf("Format = %s\n\n",Format);
  get4(Subchunk1ID);
  printf("Subchunk1ID = %s\n",Subchunk1ID);
  Subchunk1Size=get4int();
  printf("Subchunk1Size = %ld [should be 16 for 44-byte header LPCM]\n",Subchunk1Size);
    
  AudioFormat=get2int();
  printf("AudioFormat = %ld [should be 1 for LPCM]\n",AudioFormat);
    
  NumChannels=get2int();
  printf("Number of Channels = %ld\n",NumChannels);
  SampleRate=get4int();
  printf("Sample Rate = %ld Samples per sec\n",SampleRate);
  ByteRate=get4int();
  printf("Byte Rate = %ld Bytes per sec\n",ByteRate);
    
  BlockAlign=get2int();
  printf("Block Align = %ld\n\n",BlockAlign);
  BitsPerSample=get2int();
  printf("Bits per sample = %ld\n",BitsPerSample);
  BytesPerSample=BitsPerSample/8;
  printf("Bytes per sample = %ld\n\n",BytesPerSample);
    
  get4(Subchunk2ID);
  printf("Subchunk2ID = %s\n",Subchunk2ID);
  Subchunk2Size=get4int();
  printf("Subchunk2Size = %ld [Number of Data payload bytes]\n\n",Subchunk2Size);
  
  result=0;
  
  if(Subchunk2ID[0]=='d') result=1;
  if(Subchunk2ID[1]=='a') result+=1;
  if(Subchunk2ID[2]=='t') result+=1;
  if(Subchunk2ID[3]=='a') result+=1;
  
  if(result==4)
  {
    printf("WAV data start located OK\n");
    
    result=(double)Subchunk2Size/(double)ByteRate;
    pairs_per_sec=SampleRate;
    
    if(NumChannels==1)
    {
      printf("MONO Samples per sec = %ld\n",pairs_per_sec);
    }
    else
    {
      printf("Sample pairs per sec = %ld\n",pairs_per_sec);
    }
    
    
    printf("Duration = %6.4lf sec\n\n",result);
    
  }
  else
  {
    printf("Not identified as WAV data!\n");
    result=0.0;
  }
  
 fclose(infile);
    
  return result;
}




long get2int(void)
{
  long result;
  unsigned char cip1,cip2;
  
  fscanf(infile,"%c",&cip1);
  fscanf(infile,"%c",&cip2);

  result=(long)cip2;
  result=(long)cip1+256*result;

  return result;
} 

long get4int(void)
{
  long result;
  unsigned char cip1,cip2,cip3,cip4;
  
  fscanf(infile,"%c",&cip1);
  fscanf(infile,"%c",&cip2);
  fscanf(infile,"%c",&cip3);
  fscanf(infile,"%c",&cip4);
    
  result=(long)cip4;
  result=(long)cip3+256*result;
  result=(long)cip2+256*result;
  result=(long)cip1+256*result;
  
  return result;
} 
void get4(unsigned char* outstring)
{
  long isp;
  unsigned char csp;
  isp=0;
  do
  {
    fscanf(infile,"%c",&csp);
    outstring[isp]=csp;
    isp++;
  } while (isp<4);
  outstring[4]=(unsigned char)0;

}
